@@ -14,12 +14,10 @@
   limitations under the License.
 */
 
-//! urdf-viz
-//! ==================
-//!
 //! Visualize [URDF(Unified Robot Description Format)](http://wiki.ros.org/urdf) file.
 //! `urdf-viz` is written by rust-lang.
-//!
+
+#![warn(missing_debug_implementations, rust_2018_idioms)]
 
 #[cfg(feature = "assimp")]
 extern crate assimp_crate as assimp;
@@ -30,7 +28,11 @@ mod assimp_utils;
 pub mod app;
 mod errors;
 pub use errors::*;
+mod handle;
+pub use handle::*;
+#[cfg(not(target_arch = "wasm32"))]
 mod web_server;
+#[cfg(not(target_arch = "wasm32"))]
 pub use web_server::*;
 mod viewer;
 pub use viewer::*;
@@ -38,6 +40,7 @@ mod mesh;
 pub use mesh::*;
 mod urdf;
 pub use urdf::*;
+pub mod utils;
 
 // re-export
 #[doc(no_inline)]
